:- use_module(library(lists)).

perm00([], []).
perm00(List, Perm) :-
    select(First, List, Rest),  % Select an arbitrary element First, Rest remaining
    perm00(Rest, Perm0),        % Permute Rest, returning Perm0
    Perm = [First|Perm0].       % Insert the selected element, First in front of Perm0 to get Perm

perm0([], []).
perm0(List, [First|Perm0]) :-
    select(First, List, Rest),  % Select an arbitrary element First, Rest remaining
    perm0(Rest, Perm0).         % Permute Rest, to become Perm0, the tail of the output list.

perm1([], []).
perm1([H|T], Perm) :-
    perm1(T, Perm0),             % Permute T, the tail of the input, obtaining Perm0.
    select(H, Perm, Perm0).     % Insert H, the head of the input, into an arbitrary 
                                % position within Perm0 to get the output list Perm.

all_perm0(List) :-
    perm0(List, _), fail.
all_perm0(_).

all_perm1(List) :-
    perm1(List, _), fail.
all_perm1(_).

:- ensure_loaded(time0).            % time0.pl is not a module file, this is
                                    % why use_module cannot be used

:- use_module(library(between)).    % to have numlist/3

% check if the two variants deliver the same results
test0 :-
    between(7, 10, N), 
    numlist(1, N, L),
    findall(P, perm0(L,P), PsA),
    findall(P, perm1(L,P), PsB), 
    sort(PsA, PsAS),
    sort(PsB, PsBS),
    (   PsAS == PsBS -> fail
    ;   format('The two variants deliver different results for N = ~w\n', [N]),
        format('perm0: ~w\n', [PsAS]),
        format('perm: ~w\n', [PsBS])
    ).
test0 :-
    write('Same results obtained.\n').

% run the timing
test1 :-
    between(8,12,N), 
    numlist(1, N, L),
    time0('Timing perm0(N,...), N'=N, all_perm0(L)),
    time0('Timing perm1(N,...), N'=N, all_perm1(L)), nl,
    fail.
test1.


                


