:- use_module(library(lists)).
:- use_module(library(between)).

% for SWI compatibility:
foldl(Goal, List, V0, V) :-
	scanlist(Goal, List, V0, V).

foldl(Goal, List1, List2, V0, V) :-
	scanlist(Goal, List1, List2, V0, V).

foldl(Goal, List1, List2, List3, V0, V) :-
	scanlist(Goal, List1, List2, List3, V0, V).

foldl(Goal, List1, List2, List3, List4, V0, V) :-
	scanlist(Goal, List1, List2, List3, List4, V0, V).

include_local(_Pred, [], []).
include_local(Pred, [X|Xs], Ys) :-
    (   call(Pred, X) -> Ys = [X|Ys1]
    ;   Ys = Ys1
    ),
    include_local(Pred, Xs, Ys1).

maplist_local(_Pred, []).
maplist_local(Pred, [X|Xs]) :-
    call(Pred, X),
    maplist_local(Pred, Xs).

maplist_local(_Pred, [], []).
maplist_local(Pred, [X|Xs], [Y|Ys]) :-
    call(Pred, X, Y),
    maplist_local(Pred, Xs, Ys).

maplist_local(_Pred, [], [], []).
maplist_local(Pred, [X|Xs], [Y|Ys], [Z|Zs]) :-
    call(Pred, X, Y, Z), maplist_local(Pred, Xs, Ys, Zs).

hd_tl_list(H, T, [H|T]).

revapp(L, R0, R) :-
    foldl(hd_tl_list, L, R0, R).

revapp_ex1([A,B,C], R0, R) :-
    hd_tl_list(A, R0, R1),
    hd_tl_list(B, R1, R2),
    hd_tl_list(C, R2, R).

link(H, Pointer0, Pointer1) :-
    Pointer0 = [H|Pointer1].

hd_list_tl(H, [H|T], T).   % the same as link above

app(L1, L2, L3) :-
    foldl(hd_list_tl, L1, L3, L2).

app_ex1([A,B,C], R0, R) :-
    hd_list_tl(A, R0, R1), 
    hd_list_tl(B, R1, R2), 
    hd_list_tl(C, R2, R).

k_v_kv(K, V, K-V).

zip(Ks, Vs, KVs) :-
    maplist(k_v_kv, Ks, Vs, KVs).

zip_ex1([KA,KB,KC], [VA,VB,VC], [KVA,KVB,KVC]) :-
    k_v_kv(KA,VA,KVA),
    k_v_kv(KB,VB,KVB),
    k_v_kv(KC,VC,KVC).

add(A, Sum0, Sum) :-
    Sum is Sum0+A.

sumlst(L, Sum) :-
    foldl(add, L, 0, Sum).

mult(A, Prod0, Prod) :-
    Prod = Prod0*A.

fact(N, F) :-
    numlist(1, N, L),           % L = [1,2,..., N]
    foldl(mult, L, 1, FExpr),
    F is FExpr.

scp(A, B, SP0, SP) :-
    SP = SP0+A*B.

scalar_product(L1, L2, SP) :-
    foldl(scp, L1, L2, 0, SP_Expr),
    SP is SP_Expr.

scp_ex1([A1,B1,C1], [A2,B2,C2], SP) :-
    scp(A1,A2,0,  SP1),
    scp(B1,B2,SP1,SP2),
    scp(C1,C2,SP2,SP).

add_vectors(VA, VB, VC) :-            
    maplist(add, VA, VB, VC).
