:- use_module(library(lists), [permutation/2]).

% arith_expr(+L, +OpL, +Val, -Expr) :-
% Expr is an arithmetic expression containing only operators present
% in the list OpL (operators may be used 0 or more times) and
% integers given in list L (each integer has to appear exactly once),
% so that the value of the expression is Val.
arith_expr(L, OpL, Val, Expr) :-
	permutation(L, PL),              % permute the list of integers into PL
	leaves_ops_expr(PL, OpL, Expr),  % build Expr with PL as the leaves-list
	catch(Expr =:= Val, _, fail).    % check if Expr evaluates to Val, failing 
                                     % if there is a division-by-0 error.

% leaves_ops_expr(+L, +OpL, ?Expr): Expr is an arithmetic expression
% which uses operators from OpL (0 or more times each) whose leaves,
% read left-to-right form, the list L.
leaves_ops_expr(L, _OpL, Expr) :-
	L = [Expr].                      % If L is singleton, Expr is the element
leaves_ops_expr(L, OpL, Expr) :-
	append(L1, L2, L),               % Split L to nonempty L1 and L2,
	L1 \= [], L2 \= [], 
	leaves_ops_expr(L1, OpL, E1),    % generate E1 from L1 (using OpL),
	leaves_ops_expr(L2, OpL, E2),    % generate E2 from L2 (using OpL),
	member(Op, OpL),                 % select an operator Op from OpL,
	Expr =.. [Op,E1,E2].             % build the expression `E1 Op E2'.

% Solve the 1 3 4 6 problem for a given Val (value of the Expr).
solve(Val) :-
	arith_expr([1,3,4,6], [+,-,*,/], Val, Expr), write(Expr), nl, fail.
solve(_).

end_of_file.

| ?- solve(66).
(3*4-1)*6
(4*3-1)*6
6*(3*4-1)
6*(4*3-1)
yes

| ?- solve(67).
yes
