:- use_module(library(between)).

% queens_gt(N, Qs): Qs is a good placement of N queens on an NxN chessboard.
queens_gt(N, Qs):-
    placement(N, N, Qs),
    safe(Qs).

% placement(N, M, P): P is a list of length M and each element of P is between 1 and N.
placement(_, 0, []).
placement(N, M, [X|P]) :- 
    M > 0, M1 is M-1,
    between(1, N, X),
    placement(N, M1, P).

% safe(Qs): In placement Q, no pair of queens attack each other.
safe([]).
safe([Q|Qs]):-
    no_attack(Q, Qs, 1), safe(Qs).

% no_attack(Q, Qs, I): Q is the placement of the queen in row k,
% Qs lists the placements of queens in rows k+I, k+I+1, ...
% Queen in row k does not attack any of the queens listed in Qs.
no_attack(_, [], _).
no_attack(X, [Y|Ys], I):- 
    no_threat(X, Y, I), 
    J is I+1, no_attack(X, Ys, J).

% no_threat(X, Y, I): queens placed in column X of row k and in column Y of row k+I
% do not attack each other
no_threat(X, Y, I) :-
        Y =\= X, Y =\= X-I, Y =\= X+I.

end_of_file.

| ?- queens_gt(4, L).
L = [3,1,4,2] ? ;
L = [2,4,1,3] ? ;
no
| ?- queens_gt(5, L).
L = [3,1,4,2,5] ? ;
L = [2,4,1,3,5] ? ;
L = [4,1,3,5,2] ? ;
L = [1,4,2,5,3] ? ;
L = [4,2,5,3,1] ? ;
L = [1,3,5,2,4] ? ;
L = [3,5,2,4,1] ? ;
L = [2,5,3,1,4] ? ;
L = [5,3,1,4,2] ? ;
L = [5,2,4,1,3] ? ;
no
| ?- queens_gt(8, L).
L = [5,2,6,1,7,4,8,3] ? ;
L = [6,3,5,7,1,4,2,8] ? 
yes
| ?- 
