:- use_module(library(clpfd)).
:- use_module(library(lists)).

% queens_clpfd(N, Qs): Qs is a good placement of N queens on an NxN chessboard.
queens_clpfd(N, Qs):-
    placement(N, N, Qs),
    safe(Qs),
    labeling(ff,Qs).

% placement(N, M, P): P is a list of length M and each element of P is between 1 and N.
placement(_, 0, []).
placement(N, M, [X|P]) :- 
    M > 0, M1 is M-1,
    X in 1..N,
    placement(N, M1, P).

% safe(Qs): In placement Q, no pair of queens attack each other.
safe([]).
safe([Q|Qs]):-
    no_attack(Q, Qs, 1), safe(Qs).

% no_attack(Q, Qs, I): Q is the placement of the queen in row k,
% Qs lists the placements of queens in rows k+I, k+I+1, ...
% Queen in row k does not attack any of the queens listed in Qs.
no_attack(_, [], _).
no_attack(X, [Y|Ys], I):- 
    no_threat(X, Y, I), 
    J is I+1, no_attack(X, Ys, J).

% no_threat(X, Y, I): queens placed in column X of row k and in column Y of row k+I
% do not attack each other
no_threat(X, Y, I) :-
    Y #\= X, Y #\= X-I, Y #\= X+I.

