
:- use_module(library(clpfd)).

% pcount0p(L, N): L has N positive elements; 
% Version 0, pure Prolog solution.
pcount0p([], 0).
pcount0p([X|Xs], N) :-
    (    X>0 -> 
         pcount0p(Xs, N0), 
         N is N0+1
    ;    pcount0p(Xs, N)
    ).

% pcount1p(L, N): L has N positive elements; 
% Version 1, pure Prolog solution with a single recursive call
pcount1p([], 0).
pcount1p([X|Xs], N) :-
    pcount1p(Xs, N0), 

    (    X>0 -> N is N0+1
    ;           N = N0
    ).

% pcount2c(L, N): L has N positive elements; 
% Version 2, CLPFD solution, if-then-else converted to reification
pcount2c([], 0).
pcount2c([X|Xs], N) :-
    pcount2c(Xs, N0),
    X #> 0  #<=>  B,
    B #=>    N #= N0+1,
    #\ B #=> N #= N0.

% Bad pruning behaviour of pcount2c:

% | ?- pcount2c([A,B], N).
% (...)
% N in inf..sup ?                  % N could be pruned to 0..2
% yes
% | ?- pcount2c([A,B], N), A #> 4.
% (...)
% N in inf..sup ?                  % N could be pruned to 1..2


pcount3c([], 0).
pcount3c([X|Xs], N) :-
	X #> 0  #<=>  B,
	N #= N0+B,
	pcount3c(Xs, N0).
