:- use_module(library(clpfd)).
:- use_module(library(lists)).
:- use_module(library(between)).

% magic(+N,?L,Type,LabOptions,FurtherOptions): L is a magic sequence of length N.
% Type can be 'spec', 'reif', or 'glob', activating the corresponding variant
% of exactly_xxxx/3.
% LabOptions specifies the options to labeling/2.
% FurtherOptions may contain 'redundant1' and / or 'redundant2' to use the
% first and / or second type of redundant constraint. 
magic(N,L,Type,LabOptions,FurtherOptions) :-
    length(L,N),
	N1 is N-1, domain(L,0,N1),
	occurrences(L,0,L,Type),
	(
		select(redundant1,FurtherOptions,_) -> sum(L,#=,N)
	;	true
	),
	(
		select(redundant2,FurtherOptions,_) -> numlist(0,N1,Coeffs), scalar_product(Coeffs,L,#=,N)
	;	true
	),
    labeling(LabOptions,L).

% occurrences(Suffix, I, L, Type): Suffix is the suffix of L starting at position I,
% and the magic sequence constraint holds for each element of Suffix.
occurrences([],_,_,_).
occurrences([X|Suffix],I,L,Type) :-
	exactly(I,L,X,Type),
	I1 is I+1,
	occurrences(Suffix,I1,L,Type).

% exactly(I,L,X, Type): the number I occurs exactly X times in list L.
exactly(I,L,X,spec) :- exactly_spec(I,L,X).
exactly(I,L,X,reif) :- exactly_reif(I,L,X).
exactly(I,L,X,glob) :- exactly_glob(I,L,X).


% Speculative solution (uses choice points in posting the constraints):

exactly_spec(I, [I|L], M) :-
    M#>0, M1 #= M-1, exactly_spec(I, L, M1).
exactly_spec(I, [X|L], M) :-
    M#>0, X #\= I, exactly_spec(I, L, M).
exactly_spec(I, L, 0) :-
    all_nonequal(I,L).

all_nonequal(_,[]).
all_nonequal(I,[X|Xs]) :-
    I #\= X, all_nonequal(I,Xs).


% A non-speculative solution using reification:

exactly_reif(_, [], 0).
exactly_reif(I, [X|L], M) :-
    X#=I #<=> B, M#=M1+B, exactly_reif(I, L, M1).


% A non-speculative solution using a global constraint (a combinatorial library constraint):

exactly_glob(I, L, M) :-
    count(I, L, #=, M).

